Use O_CLOEXEC instead of O_RDONLY to signal loopfiles_rw() to close filehandles.
authorRob Landley <rob@landley.net>
Tue, 14 Oct 2014 19:16:34 +0000 (14:16 -0500)
committerRob Landley <rob@landley.net>
Tue, 14 Oct 2014 19:16:34 +0000 (14:16 -0500)
lib/lib.c
toys/other/truncate.c
toys/posix/cmp.c

index 7911315..19d28d2 100644 (file)
--- a/lib/lib.c
+++ b/lib/lib.c
@@ -421,8 +421,8 @@ void poke(void *ptr, uint64_t val, int size)
 // flags is O_RDONLY, stdout otherwise.  An empty argument list calls
 // function() on just stdin/stdout.
 //
// Note: read only filehandles are automatically closed when function()
// returns, but writeable filehandles must be close by function()
 void loopfiles_rw(char **argv, int flags, int permissions, int failok,
   void (*function)(int fd, char *name))
 {
@@ -441,14 +441,14 @@ void loopfiles_rw(char **argv, int flags, int permissions, int failok,
       continue;
     }
     function(fd, *argv);
-    if (flags == O_RDONLY) close(fd);
+    if (flags & O_CLOEXEC) close(fd);
   } while (*++argv);
 }
 
// Call loopfiles_rw with O_RDONLY and !failok (common case).
 void loopfiles(char **argv, void (*function)(int fd, char *name))
 {
-  loopfiles_rw(argv, O_RDONLY, 0, 0, function);
+  loopfiles_rw(argv, O_RDONLY|O_CLOEXEC, 0, 0, function);
 }
 
 // Slow, but small.
index 123ae55..d09818f 100644 (file)
@@ -35,6 +35,6 @@ void truncate_main(void)
 
   // Create files with mask rwrwrw.
   // Nonexistent files are only an error if we're supposed to create them.
-  loopfiles_rw(toys.optargs, O_WRONLY|(cr ? O_CREAT : 0), 0666, cr,
+  loopfiles_rw(toys.optargs, O_WRONLY|O_CLOEXEC|(cr ? O_CREAT : 0), 0666, cr,
     do_truncate);
 }
index 13990d4..2dae113 100644 (file)
@@ -79,6 +79,6 @@ out:
 
 void cmp_main(void)
 {
-  loopfiles_rw(toys.optargs, O_RDONLY, 0, toys.optflags&FLAG_s, do_cmp);
+  loopfiles_rw(toys.optargs, O_CLOEXEC, 0, toys.optflags&FLAG_s, do_cmp);
 }
@@ -421,8 +421,8 @@ void poke(void *ptr, uint64_t val, int size)
 // function() on just stdin/stdout.
 //
-// Note: read only filehandles are automatically closed when function()
-// returns, but writeable filehandles must be close by function()
+// Note: pass O_CLOEXEC to automatically close filehandles when function()
+// returns, otherwise filehandles must be closed by function()
 void loopfiles_rw(char **argv, int flags, int permissions, int failok,
   void (*function)(int fd, char *name))
@@ -441,14 +441,14 @@ void loopfiles_rw(char **argv, int flags, int permissions, int failok,
 }
 
-// Call loopfiles_rw with O_RDONLY and !failok (common case).
+// Call loopfiles_rw with O_RDONLY|O_CLOEXEC and !failok (common case).
 void loopfiles(char **argv, void (*function)(int fd, char *name))
 {